@@ -155,6 +155,9 @@ pub fn create_isolate_create_params<TSys: DenoLibSys>(
 
 #[cfg(any(target_os = "android", target_os = "linux"))]
 mod linux {
+  use deno_runtime::deno_node::ops::process::cgroup::CgroupVersion;
+  use deno_runtime::deno_node::ops::process::cgroup::parse_self_cgroup;
+
   /// Get memory limit with cgroup (either v1 or v2) taken into account.
   pub(super) fn get_memory_limit<TSys: crate::sys::DenoLibSys>(
     sys: &TSys,
@@ -195,104 +198,6 @@ mod linux {
 
     limit.or(system_total_memory)
   }
-
-  enum CgroupVersion<'a> {
-    V1 { cgroup_relpath: &'a str },
-    V2 { cgroup_relpath: &'a str },
-    None,
-  }
-
-  fn parse_self_cgroup(self_cgroup_content: &str) -> CgroupVersion<'_> {
-    // Initialize the cgroup version as None. This will be updated based on the parsed lines.
-    let mut cgroup_version = CgroupVersion::None;
-
-    // Iterate through each line in the cgroup content. Each line represents a cgroup entry.
-    for line in self_cgroup_content.lines() {
-      // Split the line into parts using ":" as the delimiter. The format is typically:
-      // "<hierarchy_id>:<subsystems>:<cgroup_path>"
-      let split = line.split(":").collect::<Vec<_>>();
-
-      match &split[..] {
-        // If the line specifies "memory" as the subsystem, it indicates cgroup v1 is used
-        // for memory management. Extract the relative path and update the cgroup version.
-        [_, "memory", cgroup_v1_relpath] => {
-          cgroup_version = CgroupVersion::V1 {
-            cgroup_relpath: cgroup_v1_relpath
-              .strip_prefix("/")
-              .unwrap_or(cgroup_v1_relpath),
-          };
-          // Break early since v1 explicitly manages memory, and no further checks are needed.
-          break;
-        }
-        // If the line starts with "0::", it indicates cgroup v2 is used. However, in hybrid
-        // mode, memory might still be managed by v1. Continue checking other lines to confirm.
-        ["0", "", cgroup_v2_relpath] => {
-          cgroup_version = CgroupVersion::V2 {
-            cgroup_relpath: cgroup_v2_relpath
-              .strip_prefix("/")
-              .unwrap_or(cgroup_v2_relpath),
-          };
-        }
-        _ => {}
-      }
-    }
-
-    cgroup_version
-  }
-
-  #[test]
-  fn test_parse_self_cgroup_v2() {
-    let self_cgroup = "0::/user.slice/user-1000.slice/session-3.scope";
-    let cgroup_version = parse_self_cgroup(self_cgroup);
-    assert!(matches!(
-      cgroup_version,
-      CgroupVersion::V2 { cgroup_relpath } if cgroup_relpath == "user.slice/user-1000.slice/session-3.scope"
-    ));
-  }
-
-  #[test]
-  fn test_parse_self_cgroup_hybrid() {
-    let self_cgroup = r#"12:rdma:/
-11:blkio:/user.slice
-10:devices:/user.slice
-9:cpu,cpuacct:/user.slice
-8:pids:/user.slice/user-1000.slice/session-3.scope
-7:memory:/user.slice/user-1000.slice/session-3.scope
-6:perf_event:/
-5:freezer:/
-4:net_cls,net_prio:/
-3:hugetlb:/
-2:cpuset:/
-1:name=systemd:/user.slice/user-1000.slice/session-3.scope
-0::/user.slice/user-1000.slice/session-3.scope
-"#;
-    let cgroup_version = parse_self_cgroup(self_cgroup);
-    assert!(matches!(
-      cgroup_version,
-      CgroupVersion::V1 { cgroup_relpath } if cgroup_relpath == "user.slice/user-1000.slice/session-3.scope"
-    ));
-  }
-
-  #[test]
-  fn test_parse_self_cgroup_v1() {
-    let self_cgroup = r#"11:hugetlb:/
-10:pids:/user.slice/user-1000.slice
-9:perf_event:/
-8:devices:/user.slice
-7:net_cls,net_prio:/
-6:memory:/
-5:blkio:/
-4:cpuset:/
-3:cpu,cpuacct:/
-2:freezer:/
-1:name=systemd:/user.slice/user-1000.slice/session-2.scope
-"#;
-    let cgroup_version = parse_self_cgroup(self_cgroup);
-    assert!(matches!(
-      cgroup_version,
-      CgroupVersion::V1 { cgroup_relpath } if cgroup_relpath.is_empty()
-    ));
-  }
 }
 
 #[derive(Debug, thiserror::Error, deno_error::JsError)]
