@@ -17,6 +17,7 @@ use deno_core::serde::Deserialize;
 use deno_core::serde_json;
 use deno_core::serde_json::json;
 use deno_core::url::Url;
+use deno_error::JsErrorClass;
 use deno_graph::Resolution;
 use deno_graph::ResolutionError;
 use deno_graph::SpecifierError;
@@ -48,14 +49,13 @@ use super::documents::DocumentModules;
 use super::language_server;
 use super::language_server::StateSnapshot;
 use super::performance::Performance;
-use super::tsc::TsServer;
+use super::ts_server::TsServer;
 use crate::lsp::documents::OpenDocument;
 use crate::lsp::language_server::OnceCellMap;
 use crate::lsp::lint::LspLinter;
 use crate::lsp::logging::lsp_warn;
 use crate::lsp::urls::uri_to_url;
 use crate::sys::CliSys;
-use crate::tsc::DiagnosticCategory;
 use crate::type_checker::ambient_modules_to_regex_string;
 use crate::util::path::to_percent_decoded_str;
 
@@ -876,7 +876,9 @@ fn maybe_ambient_specifier_resolution_err(
           Some(specifier.to_string())
         }
       },
-      ResolveError::ImportMap(import_map_error) => {
+      ResolveError::Other(err) => {
+        let import_map_error =
+          err.get_ref().downcast_ref::<import_map::ImportMapError>()?;
         match import_map_error.as_kind() {
           ImportMapErrorKind::UnmappedBareSpecifier(spec, _) => {
             Some(spec.clone())
@@ -891,7 +893,6 @@ fn maybe_ambient_specifier_resolution_err(
           | ImportMapErrorKind::SpecifierBacktracksAbovePrefix { .. } => None,
         }
       }
-      ResolveError::Other(..) => None,
     },
   }
 }
@@ -1318,25 +1319,9 @@ pub async fn generate_module_diagnostics(
     }
   });
 
-  let mut ts_diagnostics = ts_server
-    .get_diagnostics(snapshot.clone(), module, token)
+  let mut diagnostics = ts_server
+    .provide_diagnostics(module, snapshot.clone(), token)
     .await?;
-  let suggestion_actions_settings = snapshot
-    .config
-    .language_settings_for_specifier(&module.specifier)
-    .map(|s| s.suggestion_actions.clone())
-    .unwrap_or_default();
-  if !suggestion_actions_settings.enabled {
-    ts_diagnostics.retain(|d| {
-      d.category != DiagnosticCategory::Suggestion
-        // Still show deprecated and unused diagnostics.
-        // https://github.com/microsoft/vscode/blob/ce50bd4876af457f64d83cfd956bc916535285f4/extensions/typescript-language-features/src/languageFeatures/diagnostics.ts#L113-L114
-        || d.reports_deprecated == Some(true)
-        || d.reports_unnecessary == Some(true)
-    });
-  }
-  let mut diagnostics =
-    ts_json_to_diagnostics(ts_diagnostics, module, &snapshot.document_modules);
 
   let (deps_diagnostics, deferred_deps_diagnostics) = deps_handle
     .await
@@ -1356,9 +1341,9 @@ pub async fn generate_module_diagnostics(
     .get_or_init(async || {
       ts_server
         .get_ambient_modules(
-          snapshot.clone(),
           &module.compiler_options_key,
           module.notebook_uri.as_ref(),
+          snapshot.clone(),
           token,
         )
         .await
