@@ -65,6 +65,10 @@ impl LineIndex {
   pub fn text_content_length_utf16(&self) -> TextSize {
     self.inner.text_content_length_utf16()
   }
+
+  pub fn line_starts(&self) -> &[TextSize] {
+    self.inner.line_starts()
+  }
 }
 
 /// Compare two strings and return a vector of text edit records which are
