@@ -5,6 +5,7 @@ use std::collections::HashMap;
 use std::io::BufRead;
 use std::io::Write;
 use std::path::Path;
+use std::path::PathBuf;
 use std::process::Child;
 use std::process::Command;
 use std::process::Stdio;
@@ -13,12 +14,14 @@ use std::sync::Arc;
 use std::sync::atomic::AtomicI64;
 use std::sync::atomic::Ordering;
 
+use deno_ast::MediaType;
 use deno_config::deno_json::CompilerOptions;
 use deno_core::anyhow::anyhow;
 use deno_core::error::AnyError;
 use deno_core::parking_lot::Mutex;
 use deno_core::serde_json;
 use deno_graph::source::Resolver;
+use deno_path_util::url_from_directory_path;
 use deno_resolver::deno_json::CompilerOptionsKey;
 use indexmap::IndexSet;
 use lsp_types::Uri;
@@ -38,11 +41,11 @@ use crate::cache::DenoDir;
 use crate::http_util::HttpClientProvider;
 use crate::lsp::completions::CompletionItemData;
 use crate::lsp::documents::Document;
-use crate::lsp::documents::DocumentText;
 use crate::lsp::logging::lsp_log;
 use crate::lsp::logging::lsp_warn;
 use crate::lsp::resolver::SingleReferrerGraphResolver;
 use crate::lsp::urls::uri_to_url;
+use crate::tsc::IGNORED_DIAGNOSTIC_CODES;
 
 #[derive(Debug, Serialize, Deserialize)]
 #[serde(rename_all = "camelCase")]
@@ -158,18 +161,30 @@ impl TsGoCompilerOptions {
   }
 }
 
-#[derive(Debug, Serialize, Deserialize)]
+#[derive(Debug, Deserialize)]
 #[serde(rename_all = "camelCase")]
-struct TsGoGetDocumentParams {
-  uri: Uri,
-}
-
-#[derive(Debug, Serialize)]
-#[serde(rename_all = "camelCase")]
-struct TsGoDocumentData {
-  text: DocumentText,
-  line_starts: Vec<i32>,
-  ascii_only: bool,
+enum TsGoCallbackParams {
+  #[serde(rename_all = "camelCase")]
+  GetDocument { uri: Uri },
+  #[serde(rename_all = "camelCase")]
+  ResolveModuleName {
+    module_name: String,
+    referrer_uri: Uri,
+    import_attribute_type: Option<String>,
+    resolution_mode: deno_typescript_go_client_rust::types::ResolutionMode,
+    compiler_options_key: CompilerOptionsKey,
+  },
+  #[serde(rename_all = "camelCase")]
+  ResolveJsxImportSource {
+    compiler_options_key: CompilerOptionsKey,
+  },
+  #[serde(rename_all = "camelCase")]
+  GetPackageScopeForPath { directory_path: PathBuf },
+  #[serde(rename_all = "camelCase")]
+  GetImpliedNodeFormatForFile {
+    uri: Uri,
+    compiler_options_key: CompilerOptionsKey,
+  },
 }
 
 #[derive(Debug, Copy, Clone, Eq, PartialEq, Hash, Serialize)]
@@ -612,70 +627,52 @@ impl TsGoServerInner {
             break;
           }
         };
-        // Check if it's a request (has method) vs response (has id but no method)
         if let Some(method) = message.get("method").and_then(|m| m.as_str()) {
           let id = message.get("id");
           let params = message.get("params");
           match method {
-            "deno/host/getDocument" => {
-              let params: TsGoGetDocumentParams = match params
+            "deno/callback" => {
+              let Some(id) = id else {
+                lsp_warn!("Missing id in tsgo callback: {:#}", &message,);
+                continue;
+              };
+              let params: TsGoCallbackParams = match params
                 .and_then(|p| serde_json::from_value(p.clone()).ok())
               {
                 Some(p) => p,
                 None => {
-                  if let Some(id) = id {
-                    let response = json!({
-                      "jsonrpc": "2.0",
-                      "id": id,
-                      "error": {
-                        "code": -32602,
-                        "message": "Invalid params"
-                      }
-                    });
-                    let mut stdin = stdin_clone.lock();
-                    let _ = write_lsp_message(&mut stdin, &response);
-                  }
-                  continue;
-                }
-              };
-
-              let snapshot = snapshot_clone.lock();
-              let result =
-                snapshot.document_modules.documents.get(&params.uri).map(
-                  |doc| {
-                    let text = doc.text();
-                    TsGoDocumentData {
-                      ascii_only: text.is_ascii(),
-                      text,
-                      line_starts: doc
-                        .line_index()
-                        .line_starts()
-                        .iter()
-                        .map(|&s| u32::from(s) as i32)
-                        .collect(),
-                    }
-                  },
-                );
-
-              if let Some(id) = id {
-                let response = match result {
-                  Some(data) => json!({
-                    "jsonrpc": "2.0",
-                    "id": id,
-                    "result": data,
-                  }),
-                  None => json!({
+                  let response = json!({
                     "jsonrpc": "2.0",
                     "id": id,
                     "error": {
-                      "code": -32001,
-                      "message": "Document not found"
+                      "code": -32602,
+                      "message": "Invalid params",
                     },
-                  }),
-                };
-                let mut stdin = stdin_clone.lock();
-                let _ = write_lsp_message(&mut stdin, &response);
-              }
+                  });
+                  let mut stdin = stdin_clone.lock();
+                  let _ = write_lsp_message(&mut stdin, &response);
+                  continue;
+                }
+              };
+              let result =
+                Self::handle_callback(params, &snapshot_clone.lock());
+              let response = match result {
+                Ok(result) => json!({
+                  "jsonrpc": "2.0",
+                  "id": id,
+                  "result": result,
+                }),
+                Err(err) => json!({
+                  "jsonrpc": "2.0",
+                  "id": id,
+                  "error": {
+                    "code": -32001,
+                    "message": err.to_string(),
+                  },
+                }),
+              };
+              let mut stdin = stdin_clone.lock();
+              let _ = write_lsp_message(&mut stdin, &response);
             }
             "client/registerCapability" => {
               let response = json!({
@@ -712,12 +709,11 @@ impl TsGoServerInner {
                 });
                 let mut stdin = stdin_clone.lock();
                 let _ = write_lsp_message(&mut stdin, &response);
-              } else {
-                lsp_warn!(
-                  "Received unknown notification from tsgo: {:#}",
-                  &message
-                );
               }
+              lsp_warn!(
+                "Received unknown notification from tsgo: {:#}",
+                &message,
+              );
             }
           }
         } else if let Some(id) = message.get("id") {
@@ -744,6 +740,35 @@ impl TsGoServerInner {
       }
     });
 
+    let capabilities = {
+      let snapshot = snapshot.lock();
+      lsp::ClientCapabilities {
+        text_document: Some(lsp::TextDocumentClientCapabilities {
+          synchronization: None,
+          formatting: None,
+          range_formatting: None,
+          on_type_formatting: None,
+          publish_diagnostics: None,
+          diagnostic: Some(Default::default()),
+          ..snapshot
+            .config
+            .client_capabilities
+            .text_document
+            .clone()
+            .unwrap_or_default()
+        }),
+        workspace: Some(lsp::WorkspaceClientCapabilities {
+          symbol: snapshot
+            .config
+            .client_capabilities
+            .workspace
+            .as_ref()
+            .and_then(|w| w.symbol.clone()),
+          ..Default::default()
+        }),
+        ..Default::default()
+      }
+    };
     let initialize_request = json!({
       "jsonrpc": "2.0",
       "id": 0,
@@ -753,7 +778,7 @@ impl TsGoServerInner {
           "disablePushDiagnostics": true,
         },
         "processId": std::process::id(),
-        "capabilities": {},
+        "capabilities": capabilities,
         "rootUri": null,
         "workspaceFolders": null,
       },
@@ -797,6 +822,127 @@ impl TsGoServerInner {
     }
   }
 
+  fn handle_callback(
+    params: TsGoCallbackParams,
+    snapshot: &StateSnapshot,
+  ) -> Result<serde_json::Value, AnyError> {
+    match params {
+      TsGoCallbackParams::GetDocument { uri } => {
+        let document = snapshot
+          .document_modules
+          .documents
+          .get(&uri)
+          .ok_or_else(|| anyhow!("Document not found"))?;
+        let text = document.text();
+        Ok(json!({
+          "text": &text,
+          "lineStarts": document
+            .line_index()
+            .line_starts()
+            .iter()
+            .map(|&s| u32::from(s) as i32)
+            .collect::<Vec<_>>(),
+          "asciiOnly": text.is_ascii(),
+        }))
+      }
+      TsGoCallbackParams::ResolveModuleName {
+        module_name,
+        referrer_uri,
+        // TODO(nayeemrmn): Use this to redirect bytes/text imports.
+        import_attribute_type: _import_attribute_type,
+        resolution_mode,
+        compiler_options_key,
+      } => {
+        let referrer_module = snapshot
+          .document_modules
+          .module_for_tsgo_document(&referrer_uri, &compiler_options_key)
+          .ok_or_else(|| anyhow!("Referrer module not found"))?;
+        let Some((uri, media_type)) = snapshot
+          .document_modules
+          .resolve_dependency_document(
+          &module_name,
+          &referrer_module,
+          match resolution_mode {
+            deno_typescript_go_client_rust::types::ResolutionMode::None => {
+              ResolutionMode::Import
+            }
+            deno_typescript_go_client_rust::types::ResolutionMode::CommonJS => {
+              ResolutionMode::Require
+            }
+            deno_typescript_go_client_rust::types::ResolutionMode::ESM => {
+              ResolutionMode::Import
+            }
+          },
+        ) else {
+          return Ok(json!(null));
+        };
+        Ok(json!({
+          "uri": uri,
+          "extension": media_type.as_ts_extension(),
+        }))
+      }
+      TsGoCallbackParams::ResolveJsxImportSource {
+        compiler_options_key,
+      } => {
+        let compiler_options_data = snapshot
+          .compiler_options_resolver
+          .for_key(&compiler_options_key)
+          .unwrap();
+        let specifier = compiler_options_data
+          .jsx_import_source_config
+          .as_ref()
+          .and_then(|c| c.specifier());
+        Ok(json!(specifier))
+      }
+      TsGoCallbackParams::GetPackageScopeForPath { directory_path } => {
+        let Ok(directory_url) = url_from_directory_path(&directory_path) else {
+          return Ok(json!(null));
+        };
+        let scoped_resolver =
+          snapshot.resolver.get_scoped_resolver(Some(&directory_url));
+        let Ok(Some(package_json)) = scoped_resolver
+          .as_pkg_json_resolver()
+          .get_closest_package_json(&directory_path.join("package.json"))
+        else {
+          return Ok(json!(null));
+        };
+        Ok(json!({
+          "packageDirectoryPath": package_json.path.parent(),
+          "packageJsonText": serde_json::to_string(&package_json).unwrap(),
+        }))
+      }
+      TsGoCallbackParams::GetImpliedNodeFormatForFile {
+        uri,
+        compiler_options_key,
+      } => {
+        let referrer_module = snapshot
+          .document_modules
+          .module_for_tsgo_document(&uri, &compiler_options_key)
+          .ok_or_else(|| anyhow!("Module not found"))?;
+        let resolution_mode =
+          match (referrer_module.resolution_mode, referrer_module.media_type) {
+            (
+              _,
+              MediaType::Css
+              | MediaType::Json
+              | MediaType::Html
+              | MediaType::Sql
+              | MediaType::Wasm
+              | MediaType::SourceMap
+              | MediaType::Unknown,
+            ) => deno_typescript_go_client_rust::types::ResolutionMode::None,
+            (ResolutionMode::Import, _) => {
+              deno_typescript_go_client_rust::types::ResolutionMode::ESM
+            }
+            (ResolutionMode::Require, _) => {
+              deno_typescript_go_client_rust::types::ResolutionMode::CommonJS
+            }
+          };
+        Ok(json!(resolution_mode))
+      }
+    }
+  }
+
   async fn request<R>(
     &self,
     request: TsGoRequest,
@@ -842,6 +988,16 @@ impl TsGoServerInner {
   }
 }
 
+fn qualify_tsgo_diagnostic(diagnostic: &mut lsp::Diagnostic) {
+  diagnostic.source = Some("deno-ts".to_string());
+  if let Some(lsp::NumberOrString::Number(code)) = &diagnostic.code {
+    diagnostic.message = crate::tsc::go::maybe_rewrite_message(
+      std::mem::take(&mut diagnostic.message),
+      *code as _,
+    );
+  }
+}
+
 #[derive(Debug)]
 pub struct TsGoServer {
   deno_dir: DenoDir,
@@ -947,8 +1103,8 @@ impl TsGoServer {
     snapshot: Arc<StateSnapshot>,
     token: &CancellationToken,
   ) -> Result<lsp::DocumentDiagnosticReport, AnyError> {
-    self
-      .request(
+    let mut report = self
+      .request::<lsp::DocumentDiagnosticReport>(
         TsGoRequest::LanguageServiceMethod {
           name: "ProvideDiagnostics".to_string(),
           args: json!([&module.uri]),
@@ -958,7 +1114,22 @@ impl TsGoServer {
         snapshot,
         token,
       )
-      .await
+      .await?;
+    if let lsp::DocumentDiagnosticReport::Full(report) = &mut report {
+      report
+        .full_document_diagnostic_report
+        .items
+        .retain(|diagnostic| {
+          let Some(lsp::NumberOrString::Number(code)) = &diagnostic.code else {
+            return true;
+          };
+          !IGNORED_DIAGNOSTIC_CODES.contains(&(*code as _))
+        });
+      for diagnostic in &mut report.full_document_diagnostic_report.items {
+        qualify_tsgo_diagnostic(diagnostic);
+      }
+    }
+    Ok(report)
   }
 
   pub async fn provide_references(
