@@ -1,5 +1,6 @@
 // Copyright 2018-2026 the Deno authors. MIT license.
 
+use std::borrow::Cow;
 use std::io::IsTerminal;
 use std::io::Write;
 use std::path::Path;
@@ -8,12 +9,14 @@ use color_print::cformat;
 use color_print::cstr;
 use deno_config::deno_json::NodeModulesDirMode;
 use deno_core::anyhow::Context;
+use deno_core::anyhow::bail;
 use deno_core::error::AnyError;
 use deno_core::futures::FutureExt;
 use deno_core::serde_json::json;
 use deno_lib::args::UnstableConfig;
 use deno_npm_installer::PackagesAllowedScripts;
 use deno_runtime::WorkerExecutionMode;
+use deno_semver::jsr::JsrPackageReqReference;
 use log::info;
 
 use crate::args::DenoSubcommand;
@@ -23,25 +26,32 @@ use crate::args::InternalFlags;
 use crate::args::PermissionFlags;
 use crate::args::RunFlags;
 use crate::colors;
+use crate::util::env::resolve_cwd;
 use crate::util::temp::create_temp_node_modules_dir;
 
-pub async fn init_project(init_flags: InitFlags) -> Result<i32, AnyError> {
+pub async fn init_project(
+  flags: Flags,
+  init_flags: InitFlags,
+) -> Result<i32, AnyError> {
   if let Some(package) = &init_flags.package {
-    return init_npm(InitNpmOptions {
-      name: package,
-      args: init_flags.package_args,
-      yes: init_flags.yes,
-    })
-    .boxed_local()
-    .await;
+    if package.starts_with("jsr:") {
+      return init_jsr(package, init_flags.package_args, init_flags.yes)
+        .boxed_local()
+        .await;
+    } else if package.starts_with("npm:") {
+      return init_npm(package, init_flags.package_args, init_flags.yes)
+        .boxed_local()
+        .await;
+    } else {
+      bail!("Unknown package kind: {}", package);
+    }
   }
 
-  let cwd =
-    std::env::current_dir().context("Can't read current working directory.")?;
+  let cwd = resolve_cwd(flags.initial_cwd.as_deref())?;
   let dir = if let Some(dir) = &init_flags.dir {
     let dir = cwd.join(dir);
     std::fs::create_dir_all(&dir)?;
-    dir
+    Cow::Owned(dir)
   } else {
     cwd
   };
@@ -290,7 +300,62 @@ Deno.test(function addTest() {
   Ok(0)
 }
 
+async fn init_npm(
+  name: &str,
+  args: Vec<String>,
+  yes: bool,
+) -> Result<i32, AnyError> {
+  let script_name = npm_name_to_create_package(name);
+
+  if let Some(exit_code) = confirm_package_trust(&script_name, &args, yes)? {
+    return Ok(exit_code);
+  }
+
+  let temp_node_modules_dir = create_temp_node_modules_dir()
+    .context("Failed creating temp directory for node_modules folder.")?;
+
+  let new_flags = Flags {
+    permissions: PermissionFlags {
+      allow_all: true,
+      ..Default::default()
+    },
+    allow_scripts: PackagesAllowedScripts::All,
+    argv: args,
+    node_modules_dir: Some(NodeModulesDirMode::Auto),
+    subcommand: DenoSubcommand::Run(RunFlags {
+      script: script_name,
+      ..Default::default()
+    }),
+    reload: true,
+    internal: InternalFlags {
+      lockfile_skip_write: true,
+      root_node_modules_dir_override: Some(
+        temp_node_modules_dir.node_modules_dir_path().to_path_buf(),
+      ),
+      ..Default::default()
+    },
+    unstable_config: UnstableConfig {
+      bare_node_builtins: true,
+      sloppy_imports: true,
+      detect_cjs: true,
+      ..Default::default()
+    },
+    ..Default::default()
+  };
+  let result = crate::tools::run::run_script(
+    WorkerExecutionMode::Run,
+    new_flags.into(),
+    None,
+    None,
+    Default::default(),
+  )
+  .await;
+  drop(temp_node_modules_dir); // explicit drop for clarity
+  result
+}
+
 fn npm_name_to_create_package(name: &str) -> String {
+  let name = name.strip_prefix("npm:").unwrap_or(name);
   let mut s = "npm:".to_string();
 
   let mut scoped = false;
@@ -328,14 +393,17 @@ fn npm_name_to_create_package(name: &str) -> String {
   s
 }
 
-struct InitNpmOptions<'a> {
-  name: &'a str,
-  args: Vec<String>,
+/// Prompts the user to confirm they trust the package, unless `--yes` was
+/// passed. Returns `Ok(None)` when confirmed, or `Ok(Some(exit_code))` when
+/// the user declined (caller should return early with that code).
+fn confirm_package_trust(
+  script_name: &str,
+  args: &[String],
   yes: bool,
-}
-
-async fn init_npm(options: InitNpmOptions<'_>) -> Result<i32, AnyError> {
-  let script_name = npm_name_to_create_package(options.name);
+) -> Result<Option<i32>, AnyError> {
+  if yes {
+    return Ok(None);
+  }
 
   fn print_manual_usage(script_name: &str, args: &[String]) -> i32 {
     log::info!(
@@ -351,43 +419,48 @@ async fn init_npm(options: InitNpmOptions<'_>) -> Result<i32, AnyError> {
     1
   }
 
-  if !options.yes {
-    if std::io::stdin().is_terminal() {
-      log::info!(
-        cstr!(
-          "⚠️  Do you fully trust <y>{}</> package? Deno will invoke code from it with all permissions. Do you want to continue? <p(245)>[y/n]</>"
-        ),
-        script_name
-      );
-      loop {
-        let _ = std::io::stdout().write(b"> ")?;
-        std::io::stdout().flush()?;
-        let mut answer = String::new();
-        if std::io::stdin().read_line(&mut answer).is_ok() {
-          let answer = answer.trim().to_ascii_lowercase();
-          if answer != "y" {
-            return Ok(print_manual_usage(&script_name, &options.args));
-          } else {
-            break;
-          }
+  if std::io::stdin().is_terminal() {
+    log::info!(
+      cstr!(
+        "⚠️  Do you fully trust <y>{}</> package? Deno will invoke code from it with all permissions. Do you want to continue? <p(245)>[y/n]</>"
+      ),
+      script_name
+    );
+    loop {
+      let _ = std::io::stdout().write(b"> ")?;
+      std::io::stdout().flush()?;
+      let mut answer = String::new();
+      if std::io::stdin().read_line(&mut answer).is_ok() {
+        let answer = answer.trim().to_ascii_lowercase();
+        if answer != "y" {
+          return Ok(Some(print_manual_usage(script_name, args)));
+        } else {
+          return Ok(None);
         }
       }
-    } else {
-      return Ok(print_manual_usage(&script_name, &options.args));
     }
+  } else {
+    Ok(Some(print_manual_usage(script_name, args)))
   }
+}
 
-  let temp_node_modules_dir = create_temp_node_modules_dir()
-    .context("Failed creating temp directory for node_modules folder.")?;
+async fn init_jsr(
+  name: &str,
+  args: Vec<String>,
+  yes: bool,
+) -> Result<i32, AnyError> {
+  let script_name = jsr_name_to_create_script(name);
+
+  if let Some(exit_code) = confirm_package_trust(&script_name, &args, yes)? {
+    return Ok(exit_code);
+  }
 
   let new_flags = Flags {
     permissions: PermissionFlags {
       allow_all: true,
       ..Default::default()
     },
-    allow_scripts: PackagesAllowedScripts::All,
-    argv: options.args,
-    node_modules_dir: Some(NodeModulesDirMode::Auto),
+    argv: args,
     subcommand: DenoSubcommand::Run(RunFlags {
       script: script_name,
       ..Default::default()
@@ -395,29 +468,33 @@ async fn init_npm(options: InitNpmOptions<'_>) -> Result<i32, AnyError> {
     reload: true,
     internal: InternalFlags {
       lockfile_skip_write: true,
-      root_node_modules_dir_override: Some(
-        temp_node_modules_dir.node_modules_dir_path().to_path_buf(),
-      ),
-      ..Default::default()
-    },
-    unstable_config: UnstableConfig {
-      bare_node_builtins: true,
-      sloppy_imports: true,
-      detect_cjs: true,
       ..Default::default()
     },
     ..Default::default()
   };
-  let result = crate::tools::run::run_script(
+  crate::tools::run::run_script(
     WorkerExecutionMode::Run,
     new_flags.into(),
     None,
     None,
     Default::default(),
   )
-  .await;
-  drop(temp_node_modules_dir); // explicit drop for clarity
-  result
+  .await
+}
+
+/// Appends `/create` to the subpath of a JSR specifier.
+fn jsr_name_to_create_script(name: &str) -> String {
+  let Ok(parsed) = JsrPackageReqReference::from_str(name) else {
+    return name.to_string();
+  };
+
+  let mut inner = parsed.into_inner();
+  let new_sub_path = match inner.sub_path.as_deref() {
+    Some(sub) => format!("{}/create", sub.trim_end_matches('/')),
+    None => "create".to_string(),
+  };
+  inner.sub_path = Some(new_sub_path.as_str().into());
+  JsrPackageReqReference::new(inner).to_string()
 }
 
 fn create_json_file(
@@ -484,5 +561,45 @@ mod test {
       npm_name_to_create_package("@foo/bar@1.0.0"),
       "npm:@foo/create-bar@1.0.0".to_string()
     );
+    assert_eq!(
+      npm_name_to_create_package("npm:vite"),
+      "npm:create-vite".to_string()
+    );
+    assert_eq!(
+      npm_name_to_create_package("npm:@foo/bar@1.0.0"),
+      "npm:@foo/create-bar@1.0.0".to_string()
+    );
+  }
+
+  #[test]
+  fn jsr_name_to_create_script_test() {
+    // scoped without subpath -> appends /create
+    assert_eq!(
+      jsr_name_to_create_script("jsr:@fresh/init"),
+      "jsr:@fresh/init/create"
+    );
+    // scoped with version, no subpath -> appends /create (canonical form)
+    assert_eq!(
+      jsr_name_to_create_script("jsr:@fresh/init@2"),
+      "jsr:@fresh/init@2/create"
+    );
+    assert_eq!(
+      jsr_name_to_create_script("jsr:@fresh/init@^2.0.0"),
+      "jsr:@fresh/init@^2.0.0/create"
+    );
+    // scoped with subpath -> appends /create
+    assert_eq!(
+      jsr_name_to_create_script("jsr:@std/http/file-server"),
+      "jsr:@std/http/file-server/create"
+    );
+    assert_eq!(
+      jsr_name_to_create_script("jsr:@std/http/file-server/"),
+      "jsr:@std/http/file-server/create"
+    );
+    // scoped with subpath and version -> appends /create
+    assert_eq!(
+      jsr_name_to_create_script("jsr:@std/http@1/file-server"),
+      "jsr:@std/http@1/file-server/create"
+    );
   }
 }
