@@ -17,8 +17,8 @@ use deno_runtime::deno_io::Stdio;
 use deno_runtime::deno_io::StdioPipe;
 use deno_runtime::deno_permissions::PermissionsContainer;
 use deno_terminal::colors;
-use jupyter_protocol::messaging::StreamContent;
-use jupyter_runtime::ConnectionInfo;
+use jupyter_protocol::ConnectionInfo;
+use jupyter_protocol::StreamContent;
 use tokio::sync::mpsc;
 use tokio::sync::mpsc::UnboundedSender;
 use tokio::sync::oneshot;
