@@ -356,8 +356,7 @@ fn add_deps_from_package_json(
         }
       };
       match v {
-        deno_package_json::PackageJsonDepValue::File(_)
-        | deno_package_json::PackageJsonDepValue::JsrReq(_) => {
+        deno_package_json::PackageJsonDepValue::File(_) => {
           // ignore
         }
         deno_package_json::PackageJsonDepValue::Req(req) => {
@@ -667,6 +666,7 @@ impl DepManager {
           ext_overwrite: None,
           allow_unknown_media_types: true,
           skip_graph_roots_validation: false,
+          file_content_overrides: Default::default(),
         },
       )
       .await?;
