@@ -2,7 +2,7 @@
 
 use aws_lc_rs::rand::SecureRandom;
 use aws_lc_rs::signature::EcdsaKeyPair;
-use deno_core::ToJsBuffer;
+use deno_core::convert::Uint8Array;
 use deno_core::op2;
 use deno_core::unsync::spawn_blocking;
 use elliptic_curve::rand_core::OsRng;
@@ -39,6 +39,8 @@ pub enum GenerateKeyError {
   FailedECKeyGeneration,
   #[error("Failed to generate key")]
   FailedKeyGeneration,
+  #[error("Unsupported algorithm")]
+  UnsupportedAlgorithm,
 }
 
 // Allowlist for RSA public exponents.
@@ -67,11 +69,10 @@ pub enum GenerateKeyOptions {
   },
 }
 
-#[op2(async)]
-#[serde]
+#[op2]
 pub async fn op_crypto_generate_key(
   #[serde] opts: GenerateKeyOptions,
-) -> Result<ToJsBuffer, GenerateKeyError> {
+) -> Result<Uint8Array, GenerateKeyError> {
   let fun = || match opts {
     GenerateKeyOptions::Rsa {
       modulus_length,
@@ -159,6 +160,10 @@ fn generate_key_hmac(
     ShaHash::Sha256 => &aws_lc_rs::hmac::HMAC_SHA256,
     ShaHash::Sha384 => &aws_lc_rs::hmac::HMAC_SHA384,
     ShaHash::Sha512 => &aws_lc_rs::hmac::HMAC_SHA512,
+    // SHA3 is not supported by aws-lc-rs for HMAC
+    ShaHash::Sha3_256 | ShaHash::Sha3_384 | ShaHash::Sha3_512 => {
+      return Err(GenerateKeyError::UnsupportedAlgorithm);
+    }
   };
 
   let length = if let Some(length) = length {
