@@ -14,6 +14,8 @@ use nix::unistd::Uid;
 #[cfg(unix)]
 use nix::unistd::User;
 
+use crate::ExtNodeSys;
+
 #[derive(Debug, thiserror::Error, deno_error::JsError)]
 pub enum ProcessError {
   #[class(inherit)]
@@ -259,3 +261,151 @@ pub fn op_node_process_setuid(
 ) -> Result<(), ProcessError> {
   Err(ProcessError::NotSupported)
 }
+
+/// Returns the cgroup-constrained memory limit, or 0 if unconstrained.
+/// This matches Node.js `process.constrainedMemory()` semantics.
+#[op2(fast)]
+#[number]
+pub fn op_node_process_constrained_memory<TSys: ExtNodeSys + 'static>(
+  state: &mut OpState,
+) -> u64 {
+  #[cfg(any(target_os = "android", target_os = "linux"))]
+  {
+    let sys = state.borrow::<TSys>();
+    cgroup::cgroup_memory_limit(sys).unwrap_or(0)
+  }
+  #[cfg(not(any(target_os = "android", target_os = "linux")))]
+  {
+    let _ = state;
+    0
+  }
+}
+
+#[cfg(any(target_os = "android", target_os = "linux"))]
+pub mod cgroup {
+  pub enum CgroupVersion<'a> {
+    V1 { cgroup_relpath: &'a str },
+    V2 { cgroup_relpath: &'a str },
+    None,
+  }
+
+  pub fn parse_self_cgroup(self_cgroup_content: &str) -> CgroupVersion<'_> {
+    let mut cgroup_version = CgroupVersion::None;
+
+    for line in self_cgroup_content.lines() {
+      let split = line.split(":").collect::<Vec<_>>();
+
+      match &split[..] {
+        // cgroup v1 memory controller — takes priority, break immediately
+        [_, "memory", cgroup_v1_relpath] => {
+          cgroup_version = CgroupVersion::V1 {
+            cgroup_relpath: cgroup_v1_relpath
+              .strip_prefix("/")
+              .unwrap_or(cgroup_v1_relpath),
+          };
+          break;
+        }
+        // cgroup v2 (but keep looking for v1 memory in hybrid mode)
+        ["0", "", cgroup_v2_relpath] => {
+          cgroup_version = CgroupVersion::V2 {
+            cgroup_relpath: cgroup_v2_relpath
+              .strip_prefix("/")
+              .unwrap_or(cgroup_v2_relpath),
+          };
+        }
+        _ => {}
+      }
+    }
+
+    cgroup_version
+  }
+
+  /// Read the cgroup memory limit from the filesystem.
+  /// Returns `None` if cgroup info cannot be read or parsed.
+  pub fn cgroup_memory_limit<TSys: sys_traits::FsRead>(
+    sys: &TSys,
+  ) -> Option<u64> {
+    let self_cgroup = sys.fs_read_to_string("/proc/self/cgroup").ok()?;
+
+    match parse_self_cgroup(&self_cgroup) {
+      CgroupVersion::V1 { cgroup_relpath } => {
+        let limit_path = std::path::Path::new("/sys/fs/cgroup/memory")
+          .join(cgroup_relpath)
+          .join("memory.limit_in_bytes");
+        sys
+          .fs_read_to_string(limit_path)
+          .ok()
+          .and_then(|s| s.trim().parse::<u64>().ok())
+      }
+      CgroupVersion::V2 { cgroup_relpath } => {
+        let limit_path = std::path::Path::new("/sys/fs/cgroup")
+          .join(cgroup_relpath)
+          .join("memory.max");
+        sys
+          .fs_read_to_string(limit_path)
+          .ok()
+          .and_then(|s| s.trim().parse::<u64>().ok())
+      }
+      CgroupVersion::None => None,
+    }
+  }
+
+  #[cfg(test)]
+  mod tests {
+    use super::*;
+
+    #[test]
+    fn test_parse_self_cgroup_v2() {
+      let self_cgroup = "0::/user.slice/user-1000.slice/session-3.scope";
+      let cgroup_version = parse_self_cgroup(self_cgroup);
+      assert!(matches!(
+        cgroup_version,
+        CgroupVersion::V2 { cgroup_relpath } if cgroup_relpath == "user.slice/user-1000.slice/session-3.scope"
+      ));
+    }
+
+    #[test]
+    fn test_parse_self_cgroup_hybrid() {
+      let self_cgroup = r#"12:rdma:/
+11:blkio:/user.slice
+10:devices:/user.slice
+9:cpu,cpuacct:/user.slice
+8:pids:/user.slice/user-1000.slice/session-3.scope
+7:memory:/user.slice/user-1000.slice/session-3.scope
+6:perf_event:/
+5:freezer:/
+4:net_cls,net_prio:/
+3:hugetlb:/
+2:cpuset:/
+1:name=systemd:/user.slice/user-1000.slice/session-3.scope
+0::/user.slice/user-1000.slice/session-3.scope
+"#;
+      let cgroup_version = parse_self_cgroup(self_cgroup);
+      assert!(matches!(
+        cgroup_version,
+        CgroupVersion::V1 { cgroup_relpath } if cgroup_relpath == "user.slice/user-1000.slice/session-3.scope"
+      ));
+    }
+
+    #[test]
+    fn test_parse_self_cgroup_v1() {
+      let self_cgroup = r#"11:hugetlb:/
+10:pids:/user.slice/user-1000.slice
+9:perf_event:/
+8:devices:/user.slice
+7:net_cls,net_prio:/
+6:memory:/
+5:blkio:/
+4:cpuset:/
+3:cpu,cpuacct:/
+2:freezer:/
+1:name=systemd:/user.slice/user-1000.slice/session-2.scope
+"#;
+      let cgroup_version = parse_self_cgroup(self_cgroup);
+      assert!(matches!(
+        cgroup_version,
+        CgroupVersion::V1 { cgroup_relpath } if cgroup_relpath.is_empty()
+      ));
+    }
+  }
+}
