@@ -19,6 +19,7 @@ use base64::Engine;
 use bytes::Bytes;
 use deno_core::AsyncRefCell;
 use deno_core::AsyncResult;
+use deno_core::FromV8;
 use deno_core::OpState;
 use deno_core::RcRef;
 use deno_core::Resource;
@@ -29,6 +30,8 @@ use deno_net::UnsafelyIgnoreCertificateErrors;
 use deno_net::ops::NetError;
 use deno_net::ops::TlsHandshakeInfo;
 use deno_net::ops_tls::TlsStreamResource;
+use deno_node_crypto::x509::Certificate;
+use deno_node_crypto::x509::CertificateObject;
 use deno_tls::SocketUse;
 use deno_tls::TlsClientConfigOptions;
 use deno_tls::TlsKeys;
@@ -40,19 +43,14 @@ use rustls_tokio_stream::TlsStream;
 use rustls_tokio_stream::TlsStreamRead;
 use rustls_tokio_stream::TlsStreamWrite;
 use rustls_tokio_stream::UnderlyingStream;
-use serde::Deserialize;
 use webpki_root_certs;
 
-use super::crypto::x509::Certificate;
-use super::crypto::x509::CertificateObject;
-
 #[derive(Clone)]
 struct NodeTlsState {
   custom_ca_certs: Option<Vec<String>>,
 }
 
 #[op2]
-#[serde]
 pub fn op_get_root_certificates(state: &mut OpState) -> Vec<String> {
   if let Some(tls_state) = state.try_borrow::<NodeTlsState>()
     && let Some(certs) = &tls_state.custom_ca_certs
@@ -427,8 +425,7 @@ impl Resource for JSDuplexResource {
   }
 }
 
-#[derive(Deserialize)]
-#[serde(rename_all = "camelCase")]
+#[derive(FromV8)]
 pub struct StartJSTlsArgs {
   ca_certs: Vec<String>,
   hostname: String,
@@ -507,7 +504,7 @@ impl Resource for JSStreamTlsResource {
 #[op2]
 pub fn op_node_tls_start(
   state: Rc<RefCell<OpState>>,
-  #[serde] args: StartJSTlsArgs,
+  #[scoped] args: StartJSTlsArgs,
   #[buffer] output: &mut [u32],
 ) -> Result<(), NetError> {
   let reject_unauthorized = args.reject_unauthorized.unwrap_or(true);
@@ -587,7 +584,7 @@ pub fn op_node_tls_start(
   Ok(())
 }
 
-#[op2(async)]
+#[op2]
 #[serde]
 pub async fn op_node_tls_handshake(
   state: Rc<RefCell<OpState>>,
