@@ -3,6 +3,9 @@
 use std::cell::RefCell;
 use std::io::Write;
 
+use brotli::CompressorWriter as BrotliEncoder;
+use brotli::DecompressorWriter as BrotliDecoder;
+use deno_core::convert::Uint8Array;
 use deno_core::op2;
 use flate2::Compression;
 use flate2::write::DeflateDecoder;
@@ -41,7 +44,6 @@ unsafe impl deno_core::GarbageCollected for CompressionResource {
 }
 
 /// https://wicg.github.io/compression/#supported-formats
-#[derive(Debug)]
 enum Inner {
   DeflateDecoder(ZlibDecoder<Vec<u8>>),
   DeflateEncoder(ZlibEncoder<Vec<u8>>),
@@ -49,6 +51,23 @@ enum Inner {
   DeflateRawEncoder(DeflateEncoder<Vec<u8>>),
   GzDecoder(GzDecoder<Vec<u8>>),
   GzEncoder(GzEncoder<Vec<u8>>),
+  BrotliDecoder(Box<BrotliDecoder<Vec<u8>>>),
+  BrotliEncoder(Box<BrotliEncoder<Vec<u8>>>),
+}
+
+impl std::fmt::Debug for Inner {
+  fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+    match self {
+      Inner::DeflateDecoder(_) => write!(f, "DeflateDecoder"),
+      Inner::DeflateEncoder(_) => write!(f, "DeflateEncoder"),
+      Inner::DeflateRawDecoder(_) => write!(f, "DeflateRawDecoder"),
+      Inner::DeflateRawEncoder(_) => write!(f, "DeflateRawEncoder"),
+      Inner::GzDecoder(_) => write!(f, "GzDecoder"),
+      Inner::GzEncoder(_) => write!(f, "GzEncoder"),
+      Inner::BrotliDecoder(_) => write!(f, "BrotliDecoder"),
+      Inner::BrotliEncoder(_) => write!(f, "BrotliEncoder"),
+    }
+  }
 }
 
 #[op2]
@@ -71,17 +90,26 @@ pub fn op_compression_new(
     ("gzip", false) => {
       Inner::GzEncoder(GzEncoder::new(w, Compression::default()))
     }
+    ("brotli", true) => {
+      // 4096 is the default buffer size used by brotli crate
+      Inner::BrotliDecoder(Box::new(BrotliDecoder::new(w, 4096)))
+    }
+    ("brotli", false) => {
+      // quality level 6 and lgwin 22 are based on google's nginx default values
+      // https://github.com/google/ngx_brotli#brotli_comp_level
+      // 4096 is the default buffer size used by brotli crate
+      Inner::BrotliEncoder(Box::new(BrotliEncoder::new(w, 4096, 6, 22)))
+    }
     _ => return Err(CompressionError::UnsupportedFormat),
   };
   Ok(CompressionResource(RefCell::new(Some(inner))))
 }
 
 #[op2]
-#[buffer]
 pub fn op_compression_write(
   #[cppgc] resource: &CompressionResource,
   #[anybuffer] input: &[u8],
-) -> Result<Vec<u8>, CompressionError> {
+) -> Result<Uint8Array, CompressionError> {
   let mut inner = resource.0.borrow_mut();
   let inner = inner.as_mut().ok_or(CompressionError::ResourceClosed)?;
   let out: Vec<u8> = match &mut *inner {
@@ -115,17 +143,26 @@ pub fn op_compression_write(
       d.flush().map_err(CompressionError::Io)?;
       d.get_mut().drain(..)
     }
+    Inner::BrotliDecoder(d) => {
+      d.write_all(input).map_err(CompressionError::IoTypeError)?;
+      d.flush().map_err(CompressionError::Io)?;
+      d.get_mut().drain(..)
+    }
+    Inner::BrotliEncoder(d) => {
+      d.write_all(input).map_err(CompressionError::IoTypeError)?;
+      d.flush().map_err(CompressionError::Io)?;
+      d.get_mut().drain(..)
+    }
   }
   .collect();
-  Ok(out)
+  Ok(out.into())
 }
 
 #[op2]
-#[buffer]
 pub fn op_compression_finish(
   #[cppgc] resource: &CompressionResource,
   report_errors: bool,
-) -> Result<Vec<u8>, CompressionError> {
+) -> Result<Uint8Array, CompressionError> {
   let inner = resource
     .0
     .borrow_mut()
@@ -146,15 +183,22 @@ pub fn op_compression_finish(
     }
     Inner::GzDecoder(d) => d.finish().map_err(CompressionError::IoTypeError),
     Inner::GzEncoder(d) => d.finish().map_err(CompressionError::IoTypeError),
+    Inner::BrotliDecoder(d) => d.into_inner().map_err(|_| {
+      CompressionError::IoTypeError(std::io::Error::new(
+        std::io::ErrorKind::InvalidData,
+        "brotli decompression failed",
+      ))
+    }),
+    Inner::BrotliEncoder(d) => Ok(d.into_inner()),
   };
   match out {
     Err(err) => {
       if report_errors {
         Err(err)
       } else {
-        Ok(Vec::with_capacity(0))
+        Ok(Vec::with_capacity(0).into())
       }
     }
-    Ok(out) => Ok(out),
+    Ok(out) => Ok(out.into()),
   }
 }
