@@ -108,7 +108,7 @@ impl UnsafeWindowSurface {
   #[cppgc]
   fn new(
     state: &mut OpState,
-    #[from_v8] options: UnsafeWindowSurfaceOptions,
+    #[scoped] options: UnsafeWindowSurfaceOptions,
   ) -> Result<UnsafeWindowSurface, ByowError> {
     let instance = state
       .try_borrow::<super::Instance>()
@@ -152,7 +152,6 @@ impl UnsafeWindowSurface {
     })
   }
 
-  #[global]
   fn get_context(
     &self,
     #[this] this: v8::Global<v8::Object>,
@@ -257,17 +256,13 @@ impl<'a> FromV8<'a> for UnsafeWindowSurfaceOptions {
     let val = obj
       .get(scope, key.into())
       .ok_or_else(|| JsErrorBox::type_error("missing field 'width'"))?;
-    let width = deno_core::convert::Number::<u32>::from_v8(scope, val)
-      .map_err(JsErrorBox::from_err)?
-      .0;
+    let width = <u32>::from_v8(scope, val).map_err(JsErrorBox::from_err)?;
 
     let key = v8::String::new(scope, "height").unwrap();
     let val = obj
       .get(scope, key.into())
       .ok_or_else(|| JsErrorBox::type_error("missing field 'height'"))?;
-    let height = deno_core::convert::Number::<u32>::from_v8(scope, val)
-      .map_err(JsErrorBox::from_err)?
-      .0;
+    let height = <u32>::from_v8(scope, val).map_err(JsErrorBox::from_err)?;
 
     Ok(Self {
       system,
