@@ -34,6 +34,7 @@ use node_resolver::ResolutionMode;
 use once_cell::sync::OnceCell;
 #[cfg(not(feature = "sync"))]
 use once_cell::unsync::OnceCell;
+use serde::Deserialize;
 use serde::Serialize;
 use serde::Serializer;
 use serde_json::json;
@@ -508,6 +509,7 @@ struct MemoizedValues {
 pub struct CompilerOptionsOverrides {
   /// Skip transpiling in the loaders.
   pub no_transpile: bool,
+  pub force_check_js: bool,
   /// Base to use for the source map. This is useful when bundling
   /// and you want to make file urls relative.
   pub source_map_base: Option<Url>,
@@ -606,6 +608,13 @@ impl CompilerOptionsData {
           result.ignored_options.push(ignored);
         }
       }
+      if matches!(typ, CompilerOptionsType::Check { .. })
+        && self.overrides.force_check_js
+        && let Some(compiler_options) =
+          result.compiler_options.0.as_object_mut()
+      {
+        compiler_options.insert("checkJs".to_string(), true.into());
+      }
       if self.source_kind != CompilerOptionsSourceKind::TsConfig {
         check_warn_compiler_options(&result, &self.logged_warnings);
       }
@@ -775,6 +784,9 @@ impl CompilerOptionsData {
   }
 
   pub fn check_js(&self) -> bool {
+    if self.overrides.force_check_js {
+      return true;
+    }
     *self.memoized.check_js.get_or_init(|| {
       self
         .sources
@@ -1330,6 +1342,35 @@ impl Serialize for CompilerOptionsKey {
   }
 }
 
+impl<'de> Deserialize<'de> for CompilerOptionsKey {
+  fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
+  where
+    D: serde::Deserializer<'de>,
+  {
+    let s = String::deserialize(deserializer)?;
+    if s == "workspace-root" {
+      return Ok(Self::WorkspaceConfig(None));
+    }
+    if let Some(inner) = s
+      .strip_prefix("workspace(")
+      .and_then(|s| s.strip_suffix(')'))
+    {
+      let url = Url::parse(inner).map_err(serde::de::Error::custom)?;
+      return Ok(Self::WorkspaceConfig(Some(new_rc(url))));
+    }
+    if let Some(inner) = s
+      .strip_prefix("ts-config(")
+      .and_then(|s| s.strip_suffix(')'))
+    {
+      let i = inner.parse::<usize>().map_err(serde::de::Error::custom)?;
+      return Ok(Self::TsConfig(i));
+    }
+    Err(serde::de::Error::custom(format!(
+      "invalid CompilerOptionsKey: {s}"
+    )))
+  }
+}
+
 #[derive(Debug)]
 pub struct CompilerOptionsResolver {
   workspace_configs: FolderScopedWithUnscopedMap<CompilerOptionsData>,
